#include <stdio.h>
#include <memory.h>
#include <GrdAPI.h>

#include "writemask.h"

#define	BUFFER_SIZE 4096

// Access codes
DWORD g_dwPublic = GrdDC_DEMONVK;
DWORD g_dwPrivRD = GrdDC_DEMORDO;
DWORD g_dwPrivWR = GrdDC_DEMOPRF;
DWORD g_dwPrivMS = GrdDC_DEMOMST;

// Demo passwords of services
DWORD	dwActivatePsw	= GrdAP_GSII64_DEMO_ACTIVATION;
DWORD	dwDeactivatePsw	= GrdAP_GSII64_DEMO_DEACTIVATION;
DWORD	dwReadPsw		= GrdAP_GSII64_DEMO_READ;
DWORD	dwUpdatePsw		= GrdAP_GSII64_DEMO_UPDATE;

// ECC160 public key (40)
#define USES_ECC_PUBLIC_KEY(var)	BYTE var[40] = {0x8B,0x98,0xF2,0x40,0xD2,0x2F,0x16,0xF3,0x24,0x10,0xD0,0xD7,0x93,0xFC,0x19,0x9B,0x20,0x45,0xFF,0xF2,0xF4,0x86,0x6B,0x55,0xBB,0x19,0xC2,0xC2,0xEF,0xE4,0xC2,0x02,0x0E,0x8B,0x00,0x3D,0x35,0xCF,0xDE,0x47}
// ECC160 private key (20)
#define USES_ECC_PRIVATE_KEY(var)	BYTE var[20] = {0x12,0x31,0x82,0xC3,0x63,0xAF,0xE3,0x87,0x23,0xE9,0xAC,0x2B,0xAF,0xED,0x93,0x61,0x70,0x45,0x0C,0x45}

void FillStructures(TGrdTime &BirthTime, TGrdTime &DeadTime, TGrdLifeTime &LifeTime, TGrdFlipTime &FlipTime);

int main(int argc, char* argv[])
{
	HANDLE	hGrd;

	// Search conditions
	DWORD	dwRemoteMode			= GrdFMR_Local;	// Operation mode flags                    
	DWORD	dwFlags					= 0;			// Operation mode flags                    
	BYTE	byProg					= 0;			// Program number                          
	BYTE	byVer					= 0;			// Version                                 
	DWORD	dwID					= 0;			// Dongle ID                               
	WORD	wSN						= 0;			// Serial number                           
	WORD	wMask					= 0;			// Bit mask                                
	WORD	wType					= 0;			// Dongle type                             
	DWORD	dwModel					= GrdFMM_ALL;	// Dongle model                             
	DWORD	dwInterface				= GrdFMI_ALL;	// Dongle interface type

	// Time Structrures
	TGrdSystemTime	SysTime;
	TGrdFindInfo	FindInfo;

	TGrdTime		BirthTime;
	TGrdTime		DeadTime;
	TGrdLifeTime	LifeTime;
	TGrdFlipTime	FlipTime;

	FillStructures(BirthTime, DeadTime, LifeTime, FlipTime);

	int		nRet = 0;
	WORD	wLMS_DetSize			= 0;
	WORD	wCode_DetSize			= 0;
	WORD	wLANRes					= 0;

	BYTE	buf_dongle_header[14] = {0,0,0,0,0,0,7,0,5,0,0,0,0,0};
	BYTE	buf_mask_header[BUFFER_SIZE];
	BYTE	buf_mask[BUFFER_SIZE];

	BYTE	buf_mask_LMS_det[BUFFER_SIZE];
	BYTE	buf_mask_CODE_det[BUFFER_SIZE];

	WORD	pwModules[50];

	BYTE	pDetECC[20] = {0x7D, 0x02, 0xA0, 0x02, 0x34, 0xDA, 0x92, 0x8F, 0x41, 0x37, 0x13, 0x76, 0x45, 0x2E, 0x23, 0x72, 0xB5, 0x2C, 0x75, 0xE2};
	BYTE	pDet_16_bytes[16] = {'_', 'A', 'L', 'G', 'O', 'R', 'I', 'T', 'H', 'M', '_', '_', 'D', 'E', 'T', '_'};

	USES_ECC_PUBLIC_KEY(pPubKey4Verification);
	USES_ECC_PRIVATE_KEY(pPrivKey4KeyDecryption);

	buf_dongle_header[0] = byProg;
	buf_dongle_header[1] = byVer;
	*(WORD*)(buf_dongle_header+2) = wSN;
	*(WORD*)(buf_dongle_header+4) = wMask;

	memset(buf_mask, 0, BUFFER_SIZE);
	memset(buf_mask_header, 0, BUFFER_SIZE);

	//--------------------------------------------------------------------------------------//

	nRet = GrdStartup(dwRemoteMode);
	hGrd = GrdCreateHandle(NULL, GrdCHM_SingleThread, NULL);
	nRet = GrdSetFindMode(hGrd, dwRemoteMode, dwFlags, wType, dwID, wSN, byVer, wMask, wType, dwModel, dwInterface);
	nRet = GrdSetAccessCodes(hGrd, g_dwPublic, g_dwPrivRD, g_dwPrivWR, g_dwPrivMS);
	nRet = GrdFind(hGrd, GrdF_First, &dwID, &FindInfo);

	CWritemask wm(FindInfo.wType, FindInfo.dwModel, buf_mask, buf_mask_header);

	// wLANRes will be non-zero in case of NET dongle
	wLANRes = FindInfo.byMaxNetRes;
	//---------------------------------------------------//
	// Add a number of LMS modules
	pwModules[0] = wLANRes;
	pwModules[1] = wLANRes;
	//---------------------------------------------------//
	// LMS determinant preparation
	wm.LMS_DET_Prepare(
		buf_mask_LMS_det,			// Reserved memory
		LMS_FLAG_LSIZE_2BYTES,		// LMS flags: 2 bytes per record (otherwise 1 byte)
		1,							// Number of records (in fact, 2 modules)
		wLANRes,					// LAN Resource
		pwModules,					// WORD array of module sizes
		&wLMS_DetSize);

	if( FindInfo.wType & GrdDT_LoadableCode )
	{
		// Loadable code determinant preparation
		wm.CODE_DET_Prepare(
			buf_mask_CODE_det,		// Allocated memory
			0, "0112",				// Sequence number, letter mask of Flash
			USE_WHOLE_RAM,			// RAM start address
			USE_WHOLE_RAM,			// RAM size
			pPubKey4Verification,
			pPrivKey4KeyDecryption,
			&wCode_DetSize);		// Result determinant size
	}


	//--------------------------------------------------------------------------------------//
	// ECC160	0x0000		// Supported by Guardant Sign/Time/Code dongles
	wm.AddAlgorithm(0x0000,
		nsafl_ST_III + nsafl_ActivationSrv + nsafl_DeactivationSrv + nsafl_UpdateSrv,
		nsafh_ReadSrv + nsafh_ReadPwd, rs_algo_ECC160, GrdADS_ECC160, GrdARS_ECC160,
		dwActivatePsw, dwDeactivatePsw, dwReadPsw, dwUpdatePsw,
		NULL, NULL, NULL, NULL, 0, 0x000A, pDetECC);
	// LMS		0x0002		// Supported by Guardant network dongles
	wm.AddAlgorithm(0x0002,
		nsafl_ST_III,
		nsafh_ReadSrv, rs_algo_PI, wLMS_DetSize, 0,
		0, 0, 0, 0,
		NULL, NULL, NULL, NULL, 0, 0x000A, buf_mask_LMS_det);
	//--------------------------------------------------------------------------------------//

	printf("\nStart writing mask with number of algorithm's into dongle");

	nRet = GrdLogin(hGrd, 0xFFFFFFFF, GrdLM_PerStation);
	nRet = GrdSetWorkMode(hGrd, GrdWM_UAM, GrdWMFM_DriverAuto);

	nRet = GrdInit(hGrd);

	nRet = GrdMakeSystemTime(hGrd, 2009, 5, 5, 8, 0, 0, 0,	0, &SysTime);
	nRet = GrdSetTime(hGrd, &SysTime, NULL);	// Set dongle time (supported by Guardant Time dongles)

	nRet = GrdWrite(hGrd, 0, GrdUAM_abAlgoAddr, buf_dongle_header, NULL);
	nRet = GrdWrite(hGrd, GrdUAM_abAlgoAddr, wm.wAST_Size, buf_mask_header, NULL);
	nRet = GrdWrite(hGrd, GrdUAM_abAlgoAddr + wm.wAST_Size, wm.wMaskSize, buf_mask, NULL);

	nRet = GrdProtect(hGrd,
		GrdSAM_abyAlgoAddr + wm.wAST_Size + wm.wMaskSize,
		GrdSAM_abyAlgoAddr + wm.wAST_Size + wm.wMaskSize,
		wm.wNumberOfItems, 0x0002,		// LMS numeric name (number) is 0x0002
		0,								// Set global flags here
		NULL);

	if(nRet) 
		printf("\nFailed writing algo mask into dongle");
	else
		printf("   ...   Done.");

	nRet = GrdLogout(hGrd, 0);
	nRet = GrdCloseHandle(hGrd);
	nRet = GrdCleanup();

	//--------------------------------------------------------------------------------------//

	return 0;
}

void FillStructures(
		TGrdTime &BirthTime,
		TGrdTime &DeadTime,
		TGrdLifeTime &LifeTime,
		TGrdFlipTime &FlipTime )
{
	BirthTime.bSeconds	= 0;
	BirthTime.bMinute	= 0;
	BirthTime.bHour		= 0;
	BirthTime.bDay		= 1;
	BirthTime.bMonth	= 1;
	BirthTime.bYear		= 9;

	DeadTime.bSeconds	= 0;
	DeadTime.bMinute	= 0;
	DeadTime.bHour		= 0;
	DeadTime.bDay		= 1;
	DeadTime.bMonth		= 1;
	DeadTime.bYear		= 10;

	LifeTime.LifeTime.bSeconds	= 0;
	LifeTime.LifeTime.bMinute	= 0;
	LifeTime.LifeTime.bHour		= 0;
	LifeTime.LifeTime.bDay		= 0;
	LifeTime.LifeTime.bMonth	= 0;
	LifeTime.LifeTime.bYear		= 5;
	LifeTime.State				= 0;
	LifeTime.ReservedForEven	= 0;
}